package com.tzld.piaoquan.recommend.server.service.filter.strategy;

import com.google.common.collect.Lists;
import com.tzld.piaoquan.recommend.server.service.filter.FilterParam;
import com.tzld.piaoquan.recommend.server.service.filter.FilterStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Component
@Slf4j
public class TagStrategy implements FilterStrategy {
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Value("${video.filter.tagids:}")
    private String videoFilterTagIds;

    private Map<Long, Set<Long>> videoTagCache = new ConcurrentHashMap<>();

    @PostConstruct
    public void init() {
        initCacheByValue();
    }

    @Scheduled(cron = "0 0 0/1 * * ? ")
    public void cornInit() {
        initCacheByValue();
    }

    public void initCacheByValue() {
        long l = System.currentTimeMillis();
        if (org.apache.commons.lang.StringUtils.isNotBlank(videoFilterTagIds)) {
            Map<Long, Set<Long>> tmp = new ConcurrentHashMap<>();
            String[] tags = videoFilterTagIds.split(",");
            for (String tag : tags) {
                if (StringUtils.isBlank(tag)) {
                    continue;
                }
                Long tagId = Long.valueOf(tag);
                WxVideoTagRelExample wxVideoTagRelExample = new WxVideoTagRelExample();
                wxVideoTagRelExample.createCriteria().andTagIdEqualTo(tagId);
                List<WxVideoTagRel> wxVideoTagRels = wxVideoTagRelMapper.selectByExample(wxVideoTagRelExample);
                tmp.put(tagId,
                        wxVideoTagRels.stream().map(WxVideoTagRel::getVideoId).collect(Collectors.toSet()));
            }
            videoTagCache = tmp;
        } else {
            videoTagCache = new ConcurrentHashMap<>();
        }
        long useTime = System.currentTimeMillis() - l;
        log.info("update filter video tags data success,all data size:" +
                videoTagCache.keySet().size() + ",time:" + useTime);
    }

    @Override
    public List<Long> filter(FilterParam param) {
        // TODO 主要是涉政标签
        if (param == null
                || CollectionUtils.isEmpty(param.getVideoIds())) {
            return Collections.emptyList();
        }


        List<Long> tagIds = new ArrayList<>();
        if (StringUtils.isBlank(videoFilterTagIds)) {
            return param.getVideoIds();
        }
        String[] tags = videoFilterTagIds.split(",");
        for (String tag : tags) {
            if (Objects.isNull(tag) || Objects.equals("", tag)) {
                continue;
            }
            tagIds.add(Long.parseLong(tag));
        }

        Set<Long> retainVideoIds = new LinkedHashSet<>();
        Iterator<Long> iterator = param.getVideoIds().iterator();
        while (iterator.hasNext()) {
            Long videoId = iterator.next();
            for (Long tagId : tagIds) {
                if (!hasVideoRelTagId(videoId, tagId)) {
                    retainVideoIds.add(videoId);
                }
            }
        }


        if (CollectionUtils.isEmpty(retainVideoIds)) {
            return Collections.emptyList();
        }

        return Lists.newArrayList(retainVideoIds);
    }

    private boolean hasVideoRelTagId(Long videoId, Long tagId) {
        if (Objects.isNull(videoId) || videoId <= 0L || Objects.isNull(tagId) || tagId <= 0L) {
            return false;
        }
        Set<Long> videosByTag = videoTagCache.get(tagId);
        if (Objects.isNull(videosByTag) || videosByTag.isEmpty()) {
            return false;
        }
        return videosByTag.contains(videoId);
    }

}
